// Part of AtomViewer: AtomViewer is a tool to display and analyse
// atomistic simulations
//
// Copyright (C) 2013  ICAMS, Ruhr-Universität Bochum
//
// AtomViewer is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// AtomViewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with AtomViewer. If not, see <http://www.gnu.org/licenses/> 

package common;

import java.awt.Color;

/**
 * Table for some colors.
 * A given index return the color either for OpenGL or a Java Color
 * @author Begau
 *
 */
public class ColorTable {
	
	public enum ColorBarScheme{
		RAINBOW(new float[][]{		
		        {0.0f, 0.0f, 1.0f},
		        {0.0f, 1.0f, 1.0f},
		        {0.0f, 1.0f, 0.0f},
		        {1.0f, 1.0f, 0.0f},
		        {1.0f, 0.0f, 0.0f},
			}, "Rainbow", true),
		RAINBOW_print(new float[][]{		
		        { 12f/255f, 81f/255f,187f/255f},
		        {141f/255f,206f/255f,207f/255f},
		        {104f/255f,180f/255f, 54f/255f},
		        {252f/255f,235f/255f, 10f/255f},
		        {230f/255f, 60f/255f, 36f/255f}
			}, "Rainbow (print)", true),
		JET(new float[][]{		
				{0.0f, 0.0f, 0.5f},
		        {0.0f, 0.0f, 1.0f},
		        {0.0f, 0.5f, 1.0f},
		        {0.0f, 1.0f, 1.0f},
		        {0.5f, 1.0f, 0.5f},
		        {1.0f, 1.0f, 0.0f},
		        {1.0f, 0.5f, 0.0f},
		        {1.0f, 0.0f, 0.0f},
		        {0.5f, 0.0f, 0.0f}
			}, "Jet", true),
		HEAT(new float[][]{		
		        {0.0f, 0.0f, 0.0f},
		        {1.0f, 0.0f, 0.0f},
		        {1.0f, 1.0f, 0.0f},
		        {1.0f, 1.0f, 1.0f},
			}, "Heat", true),
		GREY(new float[][]{		
		        {0.0f, 0.0f, 0.0f},
		        {1.0f, 1.0f, 1.0f},
			}, "Grey", true),
			
		MANYCOLORS(new float[][]{		
				{0.0f, 0.0f, 0.5f},
				{1.0f, 0.0f, 0.0f},
		        {0.0f, 0.0f, 1.0f},
		        {0.0f, 1.0f, 1.0f},
		        {0.5f, 1.0f, 0.5f},
		        {1.0f, 0.5f, 0.0f},
		        {0.0f, 0.5f, 1.0f},
		        {1.0f, 1.0f, 0.0f},
		        {0.5f, 0.0f, 0.0f}
			}, "Many colors", true),
		MARTENSITE_COLORING(new float[][]{
				{1.0f, 0.0f, 0.0f},
				{1.0f, 0.46153843f, 0.0f},
				{1.0f, 0.9230769f, 0.0f},
				{0.6153846f, 1.0f, 0.0f},
				{0.15384614f, 1.0f, 0.0f},
				{0.0f, 1.0f, 0.30769253f},
				{0.0f, 1.0f, 0.76923084f},
				{0.0f, 0.7692306f, 1.0f},
				{0.0f, 0.3076923f, 1.0f},
				{0.15384626f, 0.0f, 1.0f},
				{0.61538506f, 0.0f, 1.0f},
				{1.0f, 0.0f, 0.9230771f},
				{0.5f, 0.5f, 0.5f},
				{0.5f, 0.5f, 0.5f}}, "MartensiteColors", false)
		;
		
		private final float[][] colorBar;
		private final String name;
		private final boolean generalScheme;
		
		@Override
		public String toString() {
			return this.name;
		};
		
		public boolean isGeneralScheme() {
			return generalScheme;
		}
		
		public float[][] getColorBar() {
			return colorBar;
		}
		
		ColorBarScheme(float[][] bar, String name, boolean generalScheme){
			this.colorBar = bar;
			this.name = name;
			this.generalScheme = generalScheme;
		}
	}
	
	private static ColorBarScheme colorBarScheme = ColorBarScheme.RAINBOW; 
	private static boolean colorBarSwapped = false;
	
	//Default colors
	private static final float[][] STATIC_COLORS = {
		{1f, 0.5f, 0f},
		{0f, 0f, 1f}, 
		{1f, 0f, 0f}, 
		{0f, 1f, 0f}, 
		{1f, 1f, 0f}, 
		{1f, 0f, 1f}, 
		{0f, 1f, 1f}, 
		{0f, 1f, 0.5f},
		{1f, 1f, 1f}
	};
	
	
	private static final float[][] STATIC_GRAYS = {		
		{0.9f, 0.9f, 0.9f}, 
		{0.8f, 0.8f, 0.8f}, 
		{0.7f, 0.7f, 0.7f}, 
		{0.6f, 0.6f, 0.6f}, 
		{0.5f, 0.5f, 0.5f}, 
		{0.4f, 0.4f, 0.4f}, 
		{0.3f, 0.3f, 0.3f},
		{0.2f, 0.2f, 0.2f},
		{0.1f, 0.1f, 0.1f}
	};
	
	public static ColorBarScheme getColorBarScheme() {
		return colorBarScheme;
	}
	
	public static void setColorBarScheme(ColorBarScheme colorBarScheme) {
		ColorTable.colorBarScheme = colorBarScheme;
	}
	
	public static boolean isColorBarSwapped() {
		return colorBarSwapped;
	}
	
	public static void setColorBarSwapped(boolean colorBarSwapped) {
		ColorTable.colorBarSwapped = colorBarSwapped;
	}
	
	/**
	 * Returns an OpenGl-Color as an float[3]-array for a given index
	 * @param index
	 * @return unique colors for index=0..7, a default color for everything larger
	 */
	public static float[] getGLColor(int index){
		if (index>8) index = 8;
		return STATIC_COLORS[index];		
	}
	
	/**
	 * Provides a copy of the default color array
	 * @return
	 */
	public static float[][] getGLColors(){
		float[][] f = new float[STATIC_COLORS.length][3];
		for (int i=0; i< f.length;i++){
			f[i][0] = STATIC_COLORS[i][0];
			f[i][1] = STATIC_COLORS[i][1];
			f[i][2] = STATIC_COLORS[i][2];
		}
		return f;
	}
	
	/**
	 * Returns an Java.awt.Color for a given index
	 * @param index 
	 * @return unique colors for index=0..7, a default color for everything larger
	 */
	public static Color getColor(int index){
		float[] c = ColorTable.getGLColor(index); 
		return new Color(c[0], c[1],c[2]);
	}
	
	/**
	 * Returns an OpenGl-Color (grayscales only) as an float[3]-array for a given index
	 * @param index
	 * @return unique colors for index=0..7, a default color for everything larger
	 */
	public static float[] getGrayScaleGLColor(int index){
		if (index>8) index = 8;
		return STATIC_GRAYS[index];		
	}
	
	/**
	 * Returns a color for intensity on the current color bar scheme
	 * No transparency
	 * @param value intensity between 0 and 1  
	 * @return 
	 */
	public static float[] getIntensityGLColor(float value){
		return getIntensityGLColor(value, 1f);
	}

	/**
	 * Returns a color for intensity on the current color bar scheme
	 * @param value intensity between 0 and 1  
	 * @param transparency transparency (0=full transparency, 1 fully opaque)
	 * @return  
	 */
	public static float[] getIntensityGLColor(float value, float transparency){
		if (colorBarSwapped) value = 1f-value;
		float[][] pattern = colorBarScheme.colorBar;
		int size = pattern.length;
		
		if (value>=1) return new float[]{pattern[size-1][0],pattern[size-1][1],pattern[size-1][2], transparency};
		else if (value>0 && value <1){
			float frac = 1f/(size-1);
			float[] color = new float[4];
			color[3] = transparency;
			
			int index = (int)(value/frac);
			float w = (value-index*frac)/frac;
			
			color[0] = (pattern[index+1][0]*w + pattern[index][0]*(1f-w));
			color[1] = (pattern[index+1][1]*w + pattern[index][1]*(1f-w));
			color[2] = (pattern[index+1][2]*w + pattern[index][2]*(1f-w));
			
			return color;
		} else return new float[]{pattern[0][0],pattern[0][1],pattern[0][2], transparency};
	}
	
	/**
	 * Returns a color for intensity on the current color bar scheme
	 * @param value intensity between 0 and 1  
	 * @return  
	 */
	public static float[] getIntensityGLColor(float min, float max, float value){
		float interval = max - min;
		value = (value-min)/interval;
		return getIntensityGLColor(value, 1f);
	}
	
	/**
	 * Returns a color for intensity on the current color bar scheme
	 * @param value intensity between 0 and 1 
	 * @param transparency transparency (0=full transparency, 1 fully opaque)
	 * @return
	 */
	public static float[] getIntensityGLColor(float min, float max, float value, float transparency){
		float interval = max - min;
		value = (value-min)/interval;
		return getIntensityGLColor(value, transparency);
	}
	
	/**
	 * Returns a color for intensity between blue->green->red
	 * @param value intensity between 0 and 1  
	 * @return d=>=max red; 0.5*(max-min)-min green; <=min blue 
	 */
	public static Color getIntensityColor(float min, float max, float value){
		float[] f = getIntensityGLColor(min, max, value);
		return new Color(f[0], f[1], f[2]);
	}
	
	/**
	 * Returns a color for intensity between blue->green->red
	 * @param value intensity between 0 and 1  
	 * @return d=>=1 red; 0.5 green; <=0 blue 
	 */
	public static Color getIntensityColor(float value){
		float[] f = getIntensityGLColor(value);
		return new Color(f[0], f[1], f[2]);
	}
	
	/**
	 * Creates a set of colors
	 * The colors are evenly distributed from the HSV cylinder, all colors have full saturation
	 * and no transparency 
	 * @param numberOfColors
	 * @return
	 */
	public static float[][] createColorTable(int numberOfColors){
		return createColorTable(numberOfColors, 1f);
	}
	
	/**
	 * Creates a set of colors
	 * The colors are evenly distributed from the HSV cylinder, all colors have full saturation 
	 * @param numberOfColors
	 * @param transparency
	 * @return
	 */
	public static float[][] createColorTable(int numberOfColors, float transparency){
		float[][] colors = new float[numberOfColors][];
		if (numberOfColors==0) return colors; 
		for (int i=0; i<numberOfColors; i++){
			colors[i] = HSVToRGB(((float)i/(float)(numberOfColors+1))*360f, 1f, 1f);
			colors[i][3] = transparency;
		}		
		return colors;
	}
	 
	private static float[] HSVToRGB(float h, float s, float v) {
        float[] color = new float[4];
        int i;
        float f, p, q, t;
        if (s == 0f) {
            color[0] = v; color[1] = v; color[2] = v;
            return color;
        }
        h /= 60f;
        i = (int)Math.floor(h);
        f = h - i;
        p = v * (1 - s);
        q = v * (1 - s * f);
        t = v * (1 - s * (1 - f));
        switch (i) {
            case 0:
                color[0] = v; color[1] = t; color[2] = p;
                break;
            case 1:
            	color[0] = q; color[1] = v; color[2] = p;
                break;
            case 2:
            	color[0] = p; color[1] = v; color[2] = t;
                break;
            case 3:
            	color[0] = p; color[1] = q; color[2] = v;
                break;
            case 4:
            	color[0] = t; color[1] = p; color[2] = v;
                break;
            default:
            	color[0] = v; color[1] = p; color[2] = q;
                break;
        }
        return color;
    }
}
