// Part of AtomViewer: AtomViewer is a tool to display and analyse
// atomistic simulations
//
// Copyright (C) 2013  ICAMS, Ruhr-Universität Bochum
//
// AtomViewer is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// AtomViewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with AtomViewer. If not, see <http://www.gnu.org/licenses/>

package crystalStructures;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CyclicBarrier;

import common.VectorOps;

import model.Atom;
import model.NearestNeighborBuilder;

public class L12_Ni3AlStructureExperimental extends L12_Ni3AlStructure{

	public L12_Ni3AlStructureExperimental() {
		super();
	}
	
	@Override
	protected CrystalStructure deriveNewInstance() {
		return new L12_Ni3AlStructureExperimental();
	}
	
	@Override
	protected String getIDName() {
		return "Ni3Al_experimental";
	}
	
	@Override
	public void identifyDefectAtoms(List<Atom> atoms, NearestNeighborBuilder<Atom> nnb, 
			int start, int end, CyclicBarrier barrier) {
		
		for (int i=start; i<end; i++){
			if (Thread.interrupted()) return;
			Atom a = atoms.get(i);
			a.setType(identifyAtomType(a, nnb));
		}
		try {
			barrier.await();
		} catch (Exception e) {
			if (Thread.interrupted()) return;
		}
			
		float tol = (float)(Math.cos(167)*Math.PI/180.);	
		
		for (int i=start; i<end; i++){
			if (Thread.interrupted()) return;
			Atom a = atoms.get(i);
			
			if (a.getType() == 0){
				int countPseudoTwin = 0;
				int pairs = 0;
				ArrayList<Atom> nei = nnb.getNeigh(a);
				for (int j=0; j<nei.size(); j++){
					if (nei.get(j).getType() == 8){
						countPseudoTwin++;
						float[] u = VectorOps.sub(nei.get(j), a);
						boolean pairFound = false;
						for (int k=0; k<nei.size(); k++){
							if (nei.get(k).getType() == 8){
								float[] v = VectorOps.sub(nei.get(j), a);
								if (VectorOps.angle(u, v)>tol){
									pairFound = true;
									break;
								}
							}
						}
						if (pairFound) pairs++;
					}
				}
				
				if (pairs>=2 && countPseudoTwin >= 4) a.setType(8);
			}
		}
	}
}
