// Part of AtomViewer: AtomViewer is a tool to display and analyse
// atomistic simulations
//
// Copyright (C) 2013  ICAMS, Ruhr-Universität Bochum
//
// AtomViewer is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// AtomViewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with AtomViewer. If not, see <http://www.gnu.org/licenses/> 

package model.dislocationDensity;

import javax.media.opengl.GL;
import javax.media.opengl.GL2;

import model.Coordinate;

public class CuboidVolumeElement implements VolumeElement{

	public float[] upperBound, lowerBound;
	
	public CuboidVolumeElement(float[] upperBound, float[] lowerBound) {
		if (upperBound[0]<lowerBound[0] || upperBound[1]<lowerBound[1] || upperBound[2]<lowerBound[2])
			throw new IllegalArgumentException("upper bounds must be larger than lower bounds");
		this.upperBound = upperBound;
		this.lowerBound = lowerBound;
	}

	@Override
	public boolean isInVolume(Coordinate c) {
		if (c.getX()<lowerBound[0] || c.getX()>upperBound[0]) return false;
		if (c.getY()<lowerBound[1] || c.getY()>upperBound[1]) return false;
		if (c.getZ()<lowerBound[2] || c.getZ()>upperBound[2]) return false;
		return true;
	}

	@Override
	public float getVolume() {
		return (upperBound[0]-lowerBound[0])*(upperBound[1]-lowerBound[1])*(upperBound[2]-lowerBound[2]);
	}
	
	public float[] getLowerBound() {
		return lowerBound;
	}
	
	public float[] getUpperBound() {
		return upperBound;
	}
	
	@Override
	public void render(GL2 gl, boolean picking){
		float[] u = upperBound;
		float[] l = lowerBound;
		
		gl.glBegin(GL.GL_TRIANGLES);
		gl.glNormal3f(0, 0, -1);
		gl.glVertex3f(l[0], l[1], l[2]); gl.glVertex3f(l[0], u[1], l[2]); gl.glVertex3f(u[0], u[1], l[2]);
		gl.glVertex3f(u[0], u[1], l[2]); gl.glVertex3f(u[0], l[1], l[2]); gl.glVertex3f(l[0], l[1], l[2]);
		
		gl.glNormal3f(0, 0, 1);
		gl.glVertex3f(l[0], l[1], u[2]); gl.glVertex3f(u[0], l[1], u[2]); gl.glVertex3f(u[0], u[1], u[2]);
		gl.glVertex3f(u[0], u[1], u[2]); gl.glVertex3f(l[0], u[1], u[2]); gl.glVertex3f(l[0], l[1], u[2]);
		
		gl.glNormal3f(0, -1, 0);
		gl.glVertex3f(l[0], l[1], l[2]); gl.glVertex3f(u[0], l[1], l[2]); gl.glVertex3f(u[0], l[1], u[2]);
		gl.glVertex3f(u[0], l[1], u[2]); gl.glVertex3f(l[0], l[1], u[2]); gl.glVertex3f(l[0], l[1], l[2]);
		
		gl.glNormal3f(0, 1, 0);
		gl.glVertex3f(l[0], u[1], l[2]); gl.glVertex3f(l[0], u[1], u[2]); gl.glVertex3f(u[0], u[1], u[2]);
		gl.glVertex3f(u[0], u[1], u[2]); gl.glVertex3f(u[0], u[1], l[2]); gl.glVertex3f(l[0], u[1], l[2]);
		
		gl.glNormal3f(-1, 0, 0);
		gl.glVertex3f(l[0], l[1], l[2]); gl.glVertex3f(l[0], l[1], u[2]); gl.glVertex3f(l[0], u[1], u[2]);
		gl.glVertex3f(l[0], u[1], u[2]); gl.glVertex3f(l[0], u[1], l[2]); gl.glVertex3f(l[0], l[1], l[2]);
		
		gl.glNormal3f(1, 0, 0);
		gl.glVertex3f(u[0], l[1], l[2]); gl.glVertex3f(u[0], u[1], l[2]); gl.glVertex3f(u[0], u[1], u[2]);
		gl.glVertex3f(u[0], u[1], u[2]); gl.glVertex3f(u[0], l[1], u[2]); gl.glVertex3f(u[0], l[1], l[2]);
		
		gl.glEnd();
	}
}
