// Part of AtomViewer: AtomViewer is a tool to display and analyse
// atomistic simulations
//
// Copyright (C) 2013  ICAMS, Ruhr-Universität Bochum
//
// AtomViewer is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// AtomViewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with AtomViewer. If not, see <http://www.gnu.org/licenses/> 

package model.polygrain.mesh;

import java.util.ArrayList;

import model.Coordinate;

/**
 * A brute force approach to to test if a triangle exists to a given point within a certain distance
 */
public class ClosestTriangleSearch extends ClosestTriangleSearchAlgorithm{
	private ArrayList<TriangleDataStruct> tri = new ArrayList<TriangleDataStruct>();
	
	public ClosestTriangleSearch(float threshold) {
		super(threshold);
	}
	
	public void add(Triangle t){
		tri.add(new TriangleDataStruct(t));
	}
	
	@Override
	public boolean isMeshElementWithinThreshold(Coordinate eye) {
		float closest = Float.MAX_VALUE;
		
		for (int i=0; i<tri.size();i++){
			TriangleDataStruct n = tri.get(i);
			float distToPlane = (eye.getX()*n.normal[0]+eye.getY()*n.normal[1]+eye.getZ()*n.normal[2])-n.d;
			distToPlane *= distToPlane;
			if (distToPlane < closest){
				float d = n.element.getMinSqrDist(eye, n.normal);
				if (d < closest){
					if (d < threshold)
						return true;
					closest = d;
				}
			}
		}
		return false;
	}
	
	private class TriangleDataStruct{
		public float[] normal;
		public float d;
		public Triangle element;	
		
		public TriangleDataStruct(Triangle t){
			this.normal = t.getUnitNormalVector();
			float[] p = t.getVertex().getCoord();
			d = normal[0]*p[0]+normal[1]*p[1]+normal[2]*p[2];
			this.element = t;
		}
	}
}