// Part of AtomViewer: AtomViewer is a tool to display and analyse
// atomistic simulations
//
// Copyright (C) 2013  ICAMS, Ruhr-Universität Bochum
//
// AtomViewer is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// AtomViewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with AtomViewer. If not, see <http://www.gnu.org/licenses/> 

package model.skeletonizer.processors;

import java.util.ArrayList;

import common.FastDeletableVector;

import model.ImportStates;
import model.RBV;
import model.skeletonizer.SkeletonNode;
import model.skeletonizer.Skeletonizer;

public class RBVAngleFilterPreprocessor implements SkeletonPreprocessor {

	private final float maxAngle;
	
	/**
	 * Removes all connections if the rbv's of the connected atoms deviate more than a given threshold
	 * @param maxAngle filtering threshold in degree
	 */
	public RBVAngleFilterPreprocessor(float maxAngle){
		this.maxAngle = maxAngle;
	}
	
	@Override
	public void preProcess(Skeletonizer skel) {
		if (!ImportStates.BURGERS_VECTORS.isActive()) return;
		
		final float PHI_MIN = (float) Math.cos((180f-maxAngle)*Math.PI/180.);
		final float PHI_MAX = (float) Math.cos(maxAngle*Math.PI/180.);
		
		for (SkeletonNode sn : skel.getNodes()){
			if (sn.getNeigh().size()<=2) continue;
			RBV v = sn.getMappedAtoms().get(0).getRBV();
			float l_v =  (float) Math.sqrt(v.rbv_x * v.rbv_x + v.rbv_y * v.rbv_y + v.rbv_z * v.rbv_z);
			
			ArrayList<SkeletonNode> nei = sn.getNeigh();
			int i=0;
			
			while (i<nei.size() && nei.size()>2){
				SkeletonNode n = nei.get(i);
				RBV u = n.getMappedAtoms().get(0).getRBV();
				float l_u =  (float) Math.sqrt(u.rbv_x * u.rbv_x + u.rbv_y * u.rbv_y + u.rbv_z * u.rbv_z);
				float angle = (v.rbv_x * u.rbv_x + v.rbv_y * u.rbv_y + v.rbv_z * u.rbv_z) /(l_u * l_v);
				
				if (angle < PHI_MAX && angle > PHI_MIN){
					nei.remove(i);
					n.removeNeigh(sn);
					i--;
				}
				
				i++;
			}
		}
		
		final FastDeletableVector<SkeletonNode> nodes = skel.getNodes();
		for (int i=0; i<nodes.size(); i++){
			if (nodes.get(i).getNeigh().size() == 0) {
				nodes.remove(i);
				i--;
			}
		}
	}

}
